'use strict';

exports.__esModule = true;
exports.default = playerbaseStatistics;

var _simpleStatistics = require('simple-statistics');

var stats = _interopRequireWildcard(_simpleStatistics);

var _roundTo = require('round-to');

var _roundTo2 = _interopRequireDefault(_roundTo);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

var PLAYTIME_GROUPS = [[1, 500], // 100th percentile
[500, 1000], // 80th percentile
[1000, 2000], // 60th percentile
[2000, 4000], // 40th percentile
[4000] // 20th percentile
];

var PERCENTILES = [0.995, 0.99, 0.90, 0.80, 0.50, 0.25, 0.10, 0.01];

function playerbaseStatistics(playerbase) {
  var statistics = {};
  var graphData = {};

  // Sort the playerbase by ascending value and descending playtime which makes the leaderboards
  // use descending values (highest first) and ascending playtime (lowest first)
  playerbase = playerbase.slice().sort(function (a, b) {
    return a.value - b.value || b.playtime - a.playtime;
  });

  // Add a statistic & graph for a partial playerbase
  function pushPlayerbaseData(key, partialPlayerbaseValues) {
    statistics[key] = calculateStatistics(partialPlayerbaseValues);
    graphData[key] = calculateGraphData(partialPlayerbaseValues);
  }

  // Always add a key for the whole playerbase
  pushPlayerbaseData('all', playerbase.map(function (x) {
    return x.value;
  }));

  // Add a key each per playtime group
  PLAYTIME_GROUPS.map(function (group) {
    var key = 'playtime' + (group.length === 1 ? group : group.join('to'));

    // Filter the entries that match the playtime group
    var partialPlayerbaseValues = playerbase.filter(function (x) {
      if (x.playtime < group[0] * 60 * 60) {
        return false;
      }

      if (group[1] && x.playtime > group[1] * 60 * 60) {
        return false;
      }

      return true;
    }).map(function (x) {
      return x.value;
    });

    // Calculate and push the data
    pushPlayerbaseData(key, partialPlayerbaseValues);
  });

  // Generate the leaderboard, stripping out all irrelevant information
  var leaderboard = playerbase.slice(-25).reverse().filter(function (x) {
    return x.value !== 0;
  }).map(function (x) {
    return { value: x.value, playtime: x.playtime };
  });

  // All calculations done, let's return!
  return {
    leaderboard: leaderboard,
    statistics: statistics,
    graphData: graphData
  };
}

function calculateStatistics(playerbase) {
  if (playerbase.length === 0) return {};

  // Get the basic statistics
  var statistics = {
    min: stats.minSorted(playerbase),
    max: stats.maxSorted(playerbase),
    mean: (0, _roundTo2.default)(stats.mean(playerbase), 2),
    median: (0, _roundTo2.default)(stats.medianSorted(playerbase), 2)

    // Get the value for each percentile
  };PERCENTILES.map(function (quantile) {
    var key = 'quantile' + (quantile * 100).toString().replace('.', ',');
    statistics[key] = (0, _roundTo2.default)(stats.quantileSorted(playerbase, quantile), 2);
  });

  return statistics;
}

function calculateGraphData(playerbase) {
  if (playerbase.length === 0) return [];

  // Generate 200 points for the graph, using 0.005 percentiles
  var points = [];
  for (var i = 0; i <= 1; i += 0.005) {
    points.push((0, _roundTo2.default)(stats.quantileSorted(playerbase, i), 2));
  }

  return points;
}